﻿local	_G, assert, ipairs, loadstring, math, next, pairs, pcall, print, setmetatable, string, table, tonumber, tostring, type, xpcall
=		_G, assert, ipairs, loadstring, math, next, pairs, pcall, print, setmetatable, string, table, tonumber, tostring, type, xpcall
local	CreateFrame, debugstack, GetMouseFocus, GetRealmName, GetTime, hooksecurefunc, IsAddOnLoaded, LoadAddOn, StaticPopup_Show, strsplit, UnitName
=		CreateFrame, debugstack, GetMouseFocus, GetRealmName, GetTime, hooksecurefunc, IsAddOnLoaded, LoadAddOn, StaticPopup_Show, strsplit, UnitName

local N, A = ...

BINDING_HEADER_KONG = "Kong Automatic UI Hider"
BINDING_NAME_KONG_ENABLE = "Enable UI Fading"
BINDING_NAME_KONG_DISABLE = "Disable UI Fading"
BINDING_NAME_KONG_TOGGLE = "Toggle UI Fading On/Off"
BINDING_NAME_KONG_CONFIG = "Open Config GUI"
local VERSION = "1.6a"

Kong_Frames = {}
Kong_Profiles = {}

local lastMouseFocus = nil
local lastMouseStart = nil

local MinimapParent

local CANNOT_SAVE = "Nameless frames cannot be saved."
local PROFILE_RESET = "Current profile %q has been reset."
local WELCOME = "New character detected. Type |cFF33FF99/Kong|r to configure."

StaticPopupDialogs["KONG_RESET"] = {
	text = "Are you sure you want to delete all settings from the Kong profile \"%s\"?",
	button1 = "Reset",
	button2 = CANCEL,
	OnAccept = function(self)

		Kong_Unload()
		Kong_Profiles[Kong_Settings.profile] = nil
		A.LoadSettings()

		Kong_Info(PROFILE_RESET, Kong_Settings.profile)
	end,
	showAlert = 1,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
}

-- The defaults for all triggers
local TriggerDefaults = {
	secondsIn = .2,
	secondsOut = 1,
	secondsTillOut = 0,
	alphaIn = 1,
	alphaOut = 0,
}

function Kong_Error(str, ...)
	print("|cFFFF2222Kong Error:|r "..string.format(str, ...))
end

function Kong_Warning(str, ...)
	print("|cFFFF9922Kong Warning:|r "..string.format(str, ...))
end

function Kong_Info(str, ...)
	print("|cFF33FF99Kong:|r "..string.format(str, ...))
end

local LibUSC = LibStub("LibUserSpecifiedConditions", true)
if not LibUSC then
	Kong_Error("Kong does not appear to have been installed correctly. Note: A full restart of "
			 .."World of Warcraft is required when upgrading from a previous version of Kong.")
	return
end

-- Dummy condition
local MouseoverTypeName = "Frame has Mouse Focus"
LibUSC:RegisterParameter(MouseoverTypeName, {
	type = "Truth",
	isHidden = true,
	value = false,
})

function Kong_OverrideAlpha(region, override)
	if ( Kong_Frames[region] == nil ) then return end

	if ( override ) then
		region.kongDoOverrideAlphaHook = true
		region.kongFaderAlpha = region:GetAlpha()
		region.kongFrameAlpha = region:GetAlpha()
		region.KongSetAlpha = function(region, alpha)
				region.kongDoOverrideAlphaHook = false
				region.kongFaderAlpha = alpha
				region:SetAlpha(math.min(region.kongFrameAlpha, alpha))
				region.kongDoOverrideAlphaHook = true
		end

		if ( not region.kongAlphaOverridden ) then
			hooksecurefunc(region, "SetAlpha", function(self, alpha)
					if ( self.kongDoOverrideAlphaHook ) then
						self.kongDoOverrideAlphaHook = false
						self.kongFrameAlpha = alpha
						self:SetAlpha(math.min(alpha, self.kongFaderAlpha))
						self.kongDoOverrideAlphaHook = true
					end
			end)
			region.kongAlphaOverridden = true
		end
	else
		region.kongDoOverrideAlphaHook = nil
		region.kongFaderAlpha = nil
		region.kongFrameAlpha = nil
		region.KongSetAlpha = region.SetAlpha
	end
end

function A.SetFader(region, compatibility)
	assert(Kong_Frames[region] ~= nil)
	region.KongAlphaGroup = region.KongAlphaGroup or region:CreateAnimationGroup()

	local animation = region.KongAlphaGroup:GetAnimations()
	if ( animation == nil ) then
		if ( compatibility ) then
			local animation = region.KongAlphaGroup:CreateAnimation("animation")
			function animation:SetChange(change)
				self.change = change
			end
			animation:SetScript("OnPlay", function(self)
					self.startAlpha = region:KongGetAlpha()
			end)
			animation:SetScript("OnUpdate", function(self, elapsed)
					region:SetAlpha(self.startAlpha + (self.change * self:GetSmoothProgress()))
			end)
		else
			local animation = region.KongAlphaGroup:CreateAnimation("alpha")
			animation:SetScript("OnPlay", function(self)
				self.startAlpha = region:KongGetAlpha()
			end)
			animation:SetScript("OnUpdate", function() end)
		end
	end
	Kong_Frames[region].compatibilityFader = compatibility
end

function A.CallOnFrameHeirarchy(regions, funcName, ...)
	for i, region in pairs(regions) do
		region[funcName](region, ...)
		A.CallOnFrameHeirarchy({region:GetChildren()}, funcName, ...)
	end
end

function A.DisableDraw(Frame)
	local Frames = {Frame}
	A.CallOnFrameHeirarchy(Frames, "DisableDrawLayer", "BACKGROUND")
	A.CallOnFrameHeirarchy(Frames, "DisableDrawLayer", "BORDER")
	A.CallOnFrameHeirarchy(Frames, "DisableDrawLayer", "ARTWORK")
	A.CallOnFrameHeirarchy(Frames, "DisableDrawLayer", "OVERLAY")
end

function Kong_EnableDraw(Frame)
	local Frames = {Frame}
	A.CallOnFrameHeirarchy(Frames, "EnableDrawLayer", "BACKGROUND")
	A.CallOnFrameHeirarchy(Frames, "EnableDrawLayer", "BORDER")
	A.CallOnFrameHeirarchy(Frames, "EnableDrawLayer", "ARTWORK")
	A.CallOnFrameHeirarchy(Frames, "EnableDrawLayer", "OVERLAY")

end

function Kong_Fade(region, startAlpha, endAlpha, duration, delay, finishFunc)
	region.KongAlphaGroup:Stop()

	if ( Kong_Settings.enabled and (KongConfig == nil or not KongConfig:IsShown()) ) then

		region:KongSetAlpha(startAlpha)
		local animation = region.KongAlphaGroup:GetAnimations()
		if ( animation:IsObjectType("alpha") ) then
			animation:SetFromAlpha(startAlpha)
			animation:SetToAlpha(endAlpha)
		else
			animation:SetChange(endAlpha - startAlpha)
		end
		animation:SetDuration(duration)
		
		animation:SetStartDelay(delay ~= nil and delay or 0)

		local function onFinish(self)
			region:KongSetAlpha(endAlpha)
			if ( finishFunc ~= nil ) then
				finishFunc(region)
			end
		end

		region.KongAlphaGroup:SetScript("OnFinished", onFinish)
		region.KongAlphaGroup:SetScript("OnStop", onFinish)
		region.KongAlphaGroup:Play()

		if ( duration == 0 ) then
			region.KongAlphaGroup:Stop()
		end
	end
end

function A.FadeIn(region, condition)
	condition.refs = condition.refs + 1
	if ( condition.refs == 1 ) then
		A.CalculateFadeIn(region)
	end
end

function A.CalculateFadeIn(region)
	local startAlpha = math.ceil(region:KongGetAlpha() * 100) / 100
	local endAlpha = 1
	local fastestRate = 0
	local finishFunc = nil
	local delay = 0
	
	for i, condition in ipairs(Kong_Frames[region]) do
		if ( condition.refs > 0 and condition.alphaIn >= startAlpha ) then
			if ( condition.alphaIn > startAlpha ) then
				finishFunc = A.CalculateFadeIn
			end
			local rate = (condition.alphaIn - condition.alphaOut) / condition.secondsIn
			if ( rate > fastestRate or (rate == fastestRate and condition.alphaIn > endAlpha) ) then
				endAlpha = condition.alphaIn
				fastestRate = rate
			end
		end
	end

	if ( fastestRate > 0 ) then
		local duration = (endAlpha - startAlpha) / fastestRate
		Kong_Fade(region, startAlpha, endAlpha, duration, delay, finishFunc)
		if ( Kong_Frames[region].disableDrawWhenHidden and startAlpha == 0 ) then
			Kong_EnableDraw(region)
		end
	end
end

function A.FadeOut(region, condition)
	condition.refs = condition.refs - 1
	assert(condition.refs >= 0, "ref < 0 for region: " .. tostring(region:GetName() or region))
	if ( condition.refs == 0 ) then
		local rate = (condition.alphaOut - condition.alphaIn) / condition.secondsOut
		local displacement = (condition.alphaIn - region:KongGetAlpha()) / rate
		condition.lastFadeTime = GetTime() + displacement
		Kong_CalculateFadeOut(region)
	end
end

function Kong_CalculateFadeOut(frame)
	local startAlpha = frame:KongGetAlpha()
	local endAlpha = Kong_Frames[frame].TriggerDefaults.alphaOut
	local delay = Kong_Frames[frame].TriggerDefaults.secondsTillOut
	local duration = 0
	local finishedFunc
	local chosenTrigger

	for i, trigger in ipairs(Kong_Frames[frame]) do
		if ( trigger.refs > 0 ) then
			if ( trigger.alphaIn >= startAlpha ) then return end

			endAlpha = math.max(endAlpha, trigger.alphaIn)
		elseif ( trigger.lastFadeTime ~= nil ) then
			chosenTrigger = chosenTrigger or trigger

			local now = GetTime()
			local sinceTrue = now - trigger.lastFadeTime
			local speed = (trigger.alphaOut - trigger.alphaIn) / trigger.secondsOut
			local chosenSinceTrue = now - chosenTrigger.lastFadeTime
			local chosenSpeed = (chosenTrigger.alphaOut - chosenTrigger.alphaIn) / chosenTrigger.secondsOut

			if ( sinceTrue < chosenSinceTrue or (sinceTrue == chosenSinceTrue and speed < chosenSpeed) ) then
				chosenTrigger = trigger
			end
		end
	end

	if ( chosenTrigger ~= nil ) then
		local speed = (chosenTrigger.alphaOut - chosenTrigger.alphaIn) / chosenTrigger.secondsOut
		duration = (endAlpha - startAlpha) / speed
		delay = chosenTrigger.secondsTillOut
	end

	if ( Kong_Frames[frame].disableDrawWhenHidden and endAlpha == 0 ) then
		finishedFunc = A.DisableDraw
	end

	Kong_Fade(frame, startAlpha, endAlpha, duration, delay, finishedFunc)
end

function Kong_FindGlobalIndex(region)
	local name = region:GetName()
	
	if ( name ) then
		return '["' .. name .. '"]'
	end
	
	for k, v in pairs(_G) do
		if ( v == region ) then
			local keyType = type(k)
			if ( keyType == "string" ) then
				return '["' .. k .. '"]'
			elseif ( keyType == "number" or keyType == "boolean" ) then
				return "[" .. tostring(k) .. "]"
			end
		end
	end
	
	for i, child in pairs{region:GetChildren()} do
		name = Kong_FindGlobalIndex(child)
		if ( name ) then
			return name .. ":GetParent()"
		end
	end
end

function A.CreateTrigger(Frame)

	local Trigger = {}
	setmetatable(Trigger, {__index = Kong_Frames[Frame].TriggerDefaults or TriggerDefaults})

	if Kong_Frames[Frame].TriggerDefaults then
		Trigger.refs = 0
	else
		Trigger.Frame = Frame
	end

	return Trigger
end

function Kong_RegisterFrame(region)
	Kong_Frames[region] = {}
	region.KongGetAlpha = region.GetAlpha
	region.KongSetAlpha = region.SetAlpha
	A.SetFader(region, false)
	local index = Kong_FindGlobalIndex(region)
	if ( index ~= nil ) then
		Kong_Frames[region].index = index
	else
		Kong_Frames[region].index = {}
		Kong_Warning(CANNOT_SAVE)
	end
	Kong_Frames[region].version = VERSION
	Kong_Frames[region].TriggerDefaults = A.CreateTrigger(region)
	Kong_Frames[region].Mouseover = Kong_AddTrigger(region, LibUSC:Create("Condition", LibUSC:Create(MouseoverTypeName)))
	Kong_AddTrigger(region, LibUSC:Create("Condition", LibUSC:Create("Unit in Combat")))
	Kong_AddTrigger(region, LibUSC:Create("Condition", LibUSC:Create("Unit in Vehicle")))
end

function Kong_UnregisterFrame(region)
	for i = #Kong_Frames[region], 1, -1 do
		Kong_RemoveTrigger(region, Kong_Frames[region][i])
	end
	region.KongAlphaGroup:Stop()
	Kong_OverrideAlpha(region, false)
	Kong_Fade(region, 1, 1, 0)
	region.KongGetAlpha = nil
	region.KongSetAlpha = nil
	local index = Kong_Frames[region].index
	for name, group in pairs(Kong_Settings.Groups) do
		if ( group[index] ~= nil ) then
			group[index] = nil
			if ( next(group) == nil ) then
				Kong_Settings.Groups[name] = nil
			end
		end
	end
	Kong_Frames[region] = nil
end

function A.OnConditionStateChange(region, trigger)
	if ( trigger.Condition:IsMet() ) then
		A.FadeIn(region, trigger)
	else
		A.FadeOut(region, trigger)
	end
end

function Kong_AddTrigger(region, condition)
	local trigger = A.CreateTrigger(region)
	trigger.Condition = condition
	trigger.Condition:AddListener(A.OnConditionStateChange, region, trigger)
	table.insert(Kong_Frames[region], trigger)
	if ( trigger.Condition:IsMet() ) then
		A.FadeIn(region, trigger)
	end
	return trigger
end

function Kong_RemoveTrigger(region, trigger)
	if ( trigger.Condition:IsMet() ) then
		A.FadeOut(region, trigger)
	end
	trigger.Condition:RemoveListener(A.OnConditionStateChange)
	LibUSC:Deflate(trigger.Condition)
	for i, v in ipairs(Kong_Frames[region]) do
		if ( v == trigger ) then
			table.remove(Kong_Frames[region], i)
			break
		end
	end
end

function Kong_AddFrameToGroup(region, name)
	Kong_Settings.Groups[name] = Kong_Settings.Groups[name] or {}
	local group = Kong_Settings.Groups[name]
	local index = Kong_Frames[region].index
	if ( group[index] == nil ) then
		group[index] = region
		return true
	end
end

function Kong_RemoveFrameFromGroup(region, name)
	local group = Kong_Settings.Groups[name]
	local index = Kong_Frames[region].index
	if ( group ~= nil and group[index] ~= nil ) then
		group[index] = nil
		if ( next(group) == nil ) then
			Kong_Settings.Groups[name] = nil
		end
		return true
	end
end

function A.Enable()
	Kong_Settings.enabled = true
	for Frame in pairs(Kong_Frames) do
		Kong_CalculateFadeOut(Frame)
	end
end

function A.Disable()
	for Frame in pairs(Kong_Frames) do
		if Kong_Frames[Frame].disableDrawWhenHidden and (Frame:KongGetAlpha() == 0) then
			Kong_EnableDraw(Frame)
		end
		Kong_Fade(Frame, 1, 1, 0)
	end
	Kong_Settings.enabled = false
end

function Kong_RenameFrame(region, newIndex)
	local index = Kong_Frames[region].index
	for name, group in pairs(Kong_Settings.Groups) do
		if ( group[index] ~= nil ) then
			group[index] = nil
			group[newIndex] = region
		end
	end
	Kong_Frames[region].index = newIndex
end

function A.LoadSettings()
	if ( Kong_Initialized ) then return end
	Kong_Initialized = true
	
	if ( not Kong_Settings or not Kong_Settings.profile ) then
		local playerName = string.lower( UnitName("player") )
		local playerServer = string.lower( string.gsub(GetRealmName(), " ", "_") )
		local profileName = playerName .. "_of_" .. playerServer
		
		Kong_Settings = {
			profile = profileName,
			enabled = true,
		}
	end

	Kong_Settings.profile = string.gsub(Kong_Settings.profile, " ", "_")
	
	if ( not Kong_Profiles[Kong_Settings.profile] ) then
		Kong_Profiles[Kong_Settings.profile] = {
			Frames = {},
			Groups = {},
		}
	end
	
	setmetatable(Kong_Settings, {__index = Kong_Profiles[Kong_Settings.profile]})

	Kong_Profiles[Kong_Settings.profile].version = VERSION

	A.LoadFrames()

	if ( not Kong_Settings.enabled ) then
	   Kong_Warning("Kong is currently disabled. Use |cFF33FF99/kong toggle|r to enable it.")
	end
end

function A.LoadFrames()
	for i = #Kong_Settings.Frames, 1, -1 do
		local setting = Kong_Settings.Frames[i]
		
		if ( not setting.loader ) then
			setting.loader = loadstring("return _G" .. setting.index)
		end

		local isFound, region = pcall(setting.loader)

		if (
				isFound
				and region
				and type(region.IsObjectType) == "function"
				and region:IsObjectType("Region") -- make sure it is a region; ticket #5
				and not Kong_Frames[region]
		) then
			setting.loader = nil
			Kong_Frames[region] = setting
			table.remove(Kong_Settings.Frames, i)

			setmetatable(setting.TriggerDefaults, {__index = TriggerDefaults})
			setting.TriggerDefaults.Frame = region

			region.KongGetAlpha = region.GetAlpha
			region.KongSetAlpha = region.SetAlpha
			
			if ( setting.overrideAlpha ) then
				Kong_OverrideAlpha(region, true)
			end

			A.SetFader(region, setting.compatibilityFader)
			
			for i, trigger in ipairs(setting) do
				setmetatable(trigger, {__index = setting.TriggerDefaults})
				trigger.refs = 0
				LibUSC:Inflate(trigger.Condition)
				trigger.Condition:AddListener(A.OnConditionStateChange, region, trigger)
				
				if ( MouseoverTypeName == trigger.Condition.Parameters[1]:GetName() ) then
					setting.Mouseover = trigger
				end
			end

			if ( Kong_Settings.enabled ) then
				Kong_CalculateFadeOut(region)
			end
			
			for i, trigger in ipairs(setting) do
				if ( trigger.Condition:IsMet() ) then
					A.FadeIn(region, trigger)
				end
			end
			
			for name, group in pairs(Kong_Settings.Groups) do
				if ( group[setting.index] ) then
					for i = 1, group[setting.index] do
						A.FadeIn(region, setting.Mouseover)
					end
					group[setting.index] = region
				end
			end
		end
	end
end

function Kong_Unload()
	for region, settings in pairs(Kong_Frames) do
		if ( settings.disableDrawWhenHidden and region:KongGetAlpha() == 0 ) then
			Kong_EnableDraw(region)
		end

		region.KongAlphaGroup:Stop()
		Kong_OverrideAlpha(region, false)
		Kong_Fade(region, 1, 1, 0)
		region.KongGetAlpha = nil
		region.KongSetAlpha = nil
		
		for i, trigger in ipairs(settings) do
			trigger.refs = nil
			trigger.lastFadeTime = nil
			trigger.Condition:RemoveListener(A.OnConditionStateChange)
			LibUSC:Deflate(trigger.Condition)
		end
		
		settings.loader = nil
		settings.Mouseover = nil
		settings.TriggerDefaults.Frame = nil
		
		Kong_Frames[region] = nil
		if ( type(settings.index) ~= "table" ) then
			table.insert(Kong_Settings.Frames, settings)
		end
	end
	
	if ( MinimapParent ) then
		MinimapParent.KongAlphaAnimation:GetScript("OnFinished")()
	end
	
	for name, group in pairs(Kong_Settings.Groups) do
		for index in pairs(group) do
			group[index] = 0
		end
	end
	
	Kong_Frames = {}
	lastMouseFocus = nil
	lastMouseStart = nil
	Kong_Initialized = false
end

function A.FindFirstTrackedRegion(region)
	while ( region ~= nil and Kong_Frames[region] == nil ) do
		region = not region:IsForbidden() and region:GetParent() or nil
	end
	return region
end

function A.ForEachTrackedRegion(region, func, ...)
	while ( region ~= nil ) do
		if ( Kong_Frames[region] ~= nil ) then func(region, ...) end
		region = not region:IsForbidden() and region:GetParent() or nil
	end
end

function A.MouseoverRegionFader(region, func, additive)
	local settings = Kong_Frames[region]
	for name, group in pairs(Kong_Settings.Groups) do
		if ( group[settings.index] ~= nil ) then
			for index, groupRegion in pairs(group) do
				if ( type(groupRegion) == "table" ) then -- TODO: remove value-type ambiguity
					func(groupRegion, Kong_Frames[groupRegion].Mouseover)
				else
					group[index] = groupRegion + additive
				end
			end
		end
	end
	func(region, settings.Mouseover)
end

local FRAME_FIND_PERIOD = 1.0
local frameFindTimer = 0

function A.OnUpdate(self, elapsed)
	frameFindTimer = frameFindTimer - elapsed
	if frameFindTimer <= 0 then
		frameFindTimer = FRAME_FIND_PERIOD
		A.LoadFrames()
	end

	local mouseFocus = GetMouseFocus()
	if ( mouseFocus ~= lastMouseFocus ) then
		local mouseStart = mouseFocus ~= nil and A.FindFirstTrackedRegion(mouseFocus) or nil
		if ( mouseStart ~= lastMouseStart ) then
			if ( mouseStart ~= nil ) then
				A.ForEachTrackedRegion(mouseStart, A.MouseoverRegionFader, A.FadeIn, 1)
			end
			if ( lastMouseStart ~= nil ) then
				A.ForEachTrackedRegion(lastMouseStart, A.MouseoverRegionFader, A.FadeOut, -1)
			end
			lastMouseStart = mouseStart
		end
		lastMouseFocus = mouseFocus
	end
end

function A.OnEvent(self, event, arg1, ...)

	if event == "ADDON_LOADED" then
		if arg1 == "Kong" and not Kong_Settings then
			Kong_Info(WELCOME)
		end
	elseif event == "PLAYER_LOGIN" then

		-- SexyMap custom-position workaround
		if ( IsAddOnLoaded("SexyMap") and Minimap:GetParent() == UIParent ) then
			CreateFrame("frame", "MinimapWrapper", UIParent)
			Minimap:SetParent(MinimapWrapper)
			MinimapWrapper:SetAllPoints(Minimap)
		end

		A.LoadSettings()
		
		if ( not MinimapParent ) then
			MinimapParent = Minimap:GetParent()
			if ( MinimapParent ~= UIParent ) then
				local function onPlay(self)
					if ( (self:GetToAlpha() - self:GetFromAlpha()) > 0 ) then
						Minimap:Show()
					end
				end
				
				local function onFinish(self)
					if ( MinimapParent:GetAlpha() == 0 ) then
						Minimap:Hide()
					else
						local zoom = Minimap:GetZoom()
						Minimap:SetZoom((zoom + 1) % 6)
						Minimap:SetZoom(zoom)
						Minimap:Show()
					end
				end
				
				MinimapParent:EnableMouse(true)
				MinimapParent.KongAlphaGroup = MinimapParent:CreateAnimationGroup()
				MinimapParent.KongAlphaAnimation = MinimapParent.KongAlphaGroup:CreateAnimation("alpha")
				MinimapParent.KongAlphaAnimation:SetScript("OnPlay", onPlay)
				MinimapParent.KongAlphaAnimation:SetScript("OnFinished", onFinish)
				MinimapParent.KongAlphaAnimation:SetScript("OnStop", onFinish)
			end
		end
		
		hooksecurefunc("CreateFrame", function() frameFindTimer = 0; end)
		KongUIFader:SetScript("OnUpdate", A.OnUpdate)
	elseif event == "PLAYER_LOGOUT" then
		xpcall(Kong_Unload, function(message) Kong_Settings.error = message.."\n"..debugstack() end)
	end
end

-- Fix for TukUI PetBattleHider -- TODO remove after v1.6
if TukuiPetBattleHider then
	for _,TukChild in pairs({TukuiPetBattleHider:GetChildren()}) do
		TukChild:SetParent(UIParent)
		local unit = TukChild:GetAttribute("unit")
		if unit then
			RegisterUnitWatch(TukChild, true)
			RegisterStateDriver(TukChild, "visibility", "[@"..unit..",exists,nopetbattle] show; hide")
		else
			RegisterStateDriver(TukChild, "visibility", "[nopetbattle] show; hide")
		end
	end
	UnregisterStateDriver(TukuiPetBattleHider, "visibility")
	TukuiPetBattleHider:Hide()
	TukuiPetBattleHider = UIParent
end

-- The event receiver frame
KongUIFader = CreateFrame("Frame")
KongUIFader:SetScript("OnEvent", A.OnEvent)

KongUIFader:RegisterEvent("ADDON_LOADED")
KongUIFader:RegisterEvent("PLAYER_LOGIN")
KongUIFader:RegisterEvent("PLAYER_LOGOUT")